Attribute VB_Name = "SCPI_DCPS"
'Set of functions for a standard SCPI power supply

Option Explicit

Public Sub setVoltage(volts As String, instrument As VisaComLib.FormattedIO488)
'Set output Voltage
    Dim command As String
    
    command = "VOLT " & volts
    sendCmd command, instrument
End Sub

Public Sub setCurrent(amps As String, instrument As VisaComLib.FormattedIO488)
'Set output current
    Dim command As String
    
    command = "CURR " & amps
    sendCmd command, instrument
End Sub

Public Sub outputOn(instrument As VisaComLib.FormattedIO488)
'Turn the output on
    Dim command As String
    
    command = "OUTP ON"
    sendCmd command, instrument
End Sub

Public Sub outputOff(instrument As VisaComLib.FormattedIO488)
'Turn the output off
    Dim command As String
    
    command = "OUTP OFF"
    sendCmd command, instrument
End Sub

Public Function measureVoltage(instrument As VisaComLib.FormattedIO488)
'Measure the output voltage
    Dim command As String
        
    command = "MEAS:VOLT?"
    measureVoltage = sendQry(command, instrument)
End Function

Public Function measureCurrent(instrument As VisaComLib.FormattedIO488)
'Measure the output current
    Dim command As String
        
    command = "MEAS:CURR?"
    measureCurrent = sendQry(command, instrument)
End Function

Public Function getOutputState(instrument As VisaComLib.FormattedIO488)
'Get the output state of the instrument
    Dim command As String
        
    command = "OUTP?"
    getOutputState = sendQry(command, instrument)
End Function

Public Sub MeasCurrRang(range As String, instrument As VisaComLib.FormattedIO488)
'Change current measurement range
    Dim command As String
    
    command = "SENS:CURR:RANG " & range
    sendCmd command, instrument
End Sub

Public Sub set_ov_state(state As String, instrument As VisaComLib.FormattedIO488)
'Turn over voltage protection on or off
    Dim command As String
    
    command = "VOLT:PROT:STAT " & state
    sendCmd command, instrument
End Sub

Public Sub set_ov_level(level As String, instrument As VisaComLib.FormattedIO488)
'Set the OV level
    Dim command As String
    
    command = "VOLT:PROT " & level
    sendCmd command, instrument
End Sub

Public Sub set_ocp_state(state As String, instrument As VisaComLib.FormattedIO488)
'Turn the over current protection on or off
    Dim command As String
    
    command = "CURR:PROT:STAT " & state
    sendCmd command, instrument
End Sub

Public Sub sendCmd(cmd As String, instrument As VisaComLib.FormattedIO488)
    Dim error As String
    On Error GoTo sendError
    
    instrument.WriteString cmd
    instrument.WriteString "SYST:ERR?"
    error = instrument.ReadString
    error = Left(error, Len(error) - 1)
    If error <> "+0,""No error""" Then
        MsgBox "The command that was sent resulted in the following error: " & vbCrLf & error & vbCrLf & "Please double check the command and re-enter it"
    End If
    Exit Sub
    
sendError:
    MsgBox "Lost communication with the power supply, please check your connection and restart the program"
    End
End Sub

Public Function sendQry(cmd As String, instrument As VisaComLib.FormattedIO488)
    Dim error As String
    Dim answer As String
    Dim errstring As String
    On Error GoTo QryError
    
    instrument.WriteString cmd
    answer = instrument.ReadString
    sendQry = Left(answer, Len(answer) - 1)
    Exit Function
    
QryError:
    cmd = "SYST:ERR?"
    sendCmd cmd, instrument
    error = instrument.ReadString
    error = Left(error, Len(error) - 1)
    errstring = error
    Do While error <> "+0,""No error"""
        instrument.WriteString "SYST:ERR?"
        error = instrument.ReadString
        error = Left(error, Len(error) - 1)
        If Err <> "+0,""No error""" Then errstring = errstring & vbCrLf & Err
    Loop
    MsgBox "Timeout error:" & vbCrLf & "The power supply returned the following errors: " & vbCrLf & errstring & vbCrLf & "Please check your query and try again."
End Function

Public Sub closeIO(instrument As VisaComLib.FormattedIO488)
'Close IO and free up resources
    instrument.IO.Close
End Sub

Public Function set_io(ioaddress As String, instrument As VisaComLib.FormattedIO488, Optional skipclear As Integer)
'' This subroutine will set up the VISA-COM IO Library to communicate with an instrument.
'' It will also reset the instrument and check the IDN string.
'' Inputs:
''   ioaddress - the instrument IO address as a string.  Can be LAN, USB, or GPIB
''   instrument - the instrument handle used for the VISA COM library.

    Dim answer As String
    Dim IDNstring As Variant
    On Error GoTo ioError
    
    'The following block sets up all the Instrument IO
    Set ioMgr = New AgilentRMLib.SRMCls
    Set instrument = New VisaComLib.FormattedIO488
    Set instrument.IO = ioMgr.Open(ioaddress)
    instrument.IO.Timeout = 5000
    instrument.FlushRead
    
    'This will make sure that you are communicating properly
    instrument.WriteString "*IDN?"
    answer = instrument.ReadString
    modeln = get_modelN(answer)
    
    MsgBox "The instrument you are communicating with returns the following *IDN? string:" & vbCrLf & answer
    
    If skipclear = 0 Then
        instrument.WriteString "*RST;*CLS"
    End If
    
    set_io = True
    Exit Function
    
ioError:
    MsgBox "Set IO error:" & vbCrLf & Err.Description
    set_io = False
End Function

Public Function get_modelN(IDN As String)
'Strip model number out of the IDN string
    Dim data() As String
    Dim model As String
    
    data = Split(IDN, ",")
    model = data(1)
    get_modelN = model
    
    
End Function

Public Function readError(instrument As VisaComLib.FormattedIO488)
'Read instrument errors
    Dim command As String
    
    command = "SYST:ERR?"
    readError = sendQry(command, instrument)
End Function
